@@ -16,7 +16,7 @@ import pprint
 from .chapter import Chapter
 from .helpers import (get_json_data, chunk, RateLimitedSession,
                       gather_with_semaphore, safe_to_int, horizontal_rule,
-                      find_int_between, parse_range_input, _Getch)
+                      find_int_between, parse_range_input, _Getch, sep_num_and_str)
 from .filesys import FileSys
 from .config import mangodl_config
 
@@ -24,6 +24,9 @@ from .config import mangodl_config
 import logging
 logger = logging.getLogger(__name__)
 
+# get ARGS to access ARGS.url
+from .cli import ARGS
+
 # set up logging prefixes for use in tqdm.tqdm.write
 WARNING_PREFIX = f'{__name__} | [WARNING]: '
 ERROR_PREFIX = f'{__name__} | [ERROR]: '
@@ -84,9 +87,10 @@ class Manga:
                           fs: FileSys,
                           lang: str,
                           saver: bool,
-                          rate_limit: Optional[int],
+                          rate_limit: int,
                           no_volume: bool,
-                          vol_len: int):
+                          vol_len: int,
+                          no_prompt: bool = False):
         """
         Saves all chapters into a folder.
 
@@ -97,12 +101,14 @@ class Manga:
             Manga language.
         saver : bool
             Lower quality images if set to True.
-        rate_limit : int, optional
-            Used to construct a `RateLimitedSession` instance. By default, no limit is placed.
+        rate_limit : int
+            Used to construct a `RateLimitedSession` instance.
         no_volume : bool
             Automatically converts into volume if False.
         vol_len : int
             Default length per volume if not provided by mangadex.
+        no_prompt : bool, default False
+            If set to True, will download every chapter found without prompting user.
 
         Returns
         -------
@@ -150,9 +156,7 @@ class Manga:
         async def main_download() -> Awaitable:
             downloads = []
             async with aiohttp.ClientSession() as session:
-                if rate_limit:
-                    session = RateLimitedSession(
-                        session, rate_limit, rate_limit)
+                session = RateLimitedSession(session, rate_limit, rate_limit)
                 for raw_ch in self.s_downloads:
                     downloads.append(
                         check_server_and_download(session, raw_ch))
@@ -167,33 +171,48 @@ class Manga:
 
         if not self.p_downloads:
             logger.critical(f'no chapters found for {self.title}')
+            if ARGS.url:
+                logger.info(f'quitting application')
+                sys.exit()
             from .mangodl import next_manga
             next_manga()
 
-        # prompt user here
-        self._display_chs()
+        if no_prompt:
+            self.s_downloads = self.p_downloads
+        else:
+            # show some info to the user and get input
+            self._display_chs()
+        #
         # file system stuff here
         fs.setup_folders()
+        #
         # download all chapters
         asyncio.run(main_download())
         logger.info('all chapters downloaded (ᵔᴥᵔ)')
-        # ensure every chapter is assigned a volume
+        #
+        # check and assign volumes
         if not no_volume:
             self._compile_volume_info(vol_len)
 
     def _display_chs(self):
-        """Print out some info about the chapters found for user and solicits user input
+        """Print out some info about the chapters found and solicits user input
         before commencing download."""
-        ch_nums = [safe_to_int(chap['chapter']) for chap in self.p_downloads]
+        #
+        # just take away the chapters with no chapter numbers first
+        ch_nums, ch_str_nums = sep_num_and_str(
+            [safe_to_int(chap['chapter']) for chap in self.p_downloads])
         ch_nums.sort()
-        self.missing = find_int_between(ch_nums)
+
         horizontal_rule()
-        print(f'Found {len(ch_nums)} chapter(s) for {self.title}')
+        print(f'Found {len(self.p_downloads)} chapter(s) for {self.title}')
         if len(ch_nums) == 1:
             print(f'Chapter {ch_nums[0]} can be downloaded.')
         else:
             print(f'    First chapter: {ch_nums[0]}')
             print(f'    Last chapter: {ch_nums[-1]}')
+        #
+        # check for potentially missing chapters
+        self.missing = find_int_between(ch_nums)
         if self.missing:
             n = len(self.missing)
             if n == 1:
@@ -205,12 +224,19 @@ class Manga:
                 pprint.pprint([ch for ch in self.missing], compact=True,
                               width=min(len(self.missing), 80))
                 print('\033[0m', end='')
-
+        #
+        # prompt user for download range
         selection = self._get_download_range(ch_nums)
 
-        # selected downloads
         self.s_downloads = [
             ch for ch in self.p_downloads if ch['chapter'] in selection]
+        #
+        # now deal with those without any chapter numbers
+        if ch_str_nums:
+            nameless_chs = [ch for ch in self.p_downloads if isinstance(
+                safe_to_int(ch['chapter']), str)]
+            horizontal_rule()
+            self.handle_nameless(nameless_chs)
 
         # if not all chapters were selected
         if len(self.s_downloads) < len(self.p_downloads):
@@ -234,7 +260,7 @@ class Manga:
                             logger.info(
                                 f'chapter {ch_num} queued for download')
                         else:
-                            logger.warning(
+                            logger.info(
                                 f'chapter {ch_num} will not be downloaded')
                 if not s:
                     # nothing selected!
@@ -246,26 +272,45 @@ class Manga:
                 return collect_range_input()
 
         print('Which chapters to download?')
-        # print('    ↓ or just use one of these options ↓')
-        # consider adding more options
-        print('[a] - download all    [r] - select custom range    [s] - search another manga    [q] - quit app')
-        c = getch()
 
-        if c.lower() == 'a':
-            return {str(_) for _ in ch_nums}
-        elif c.lower() == 'r':
-            logger.info(f'input {c} - select custom range')
-            collect_range_input()
-        elif c.lower() == 's':
-            logger.warning(f'input {c} - abandoning the manga {self.title}')
-            from .mangodl import search_another
-            search_another()
-        elif c.lower() == 'q':
-            logger.info(f'input {c} - quitting application')
-            sys.exit()
+        if ARGS.url:
+            # don't print the 'search another manga' option
+            print(
+                '[a] - download all    [r] - select custom range    [q] - quit app')
+            c = getch()
+
+            if c.lower() == 'a':
+                return {str(_) for _ in ch_nums}
+            elif c.lower() == 'r':
+                logger.info(f'input {c} - select custom range')
+                collect_range_input()
+            elif c.lower() == 'q':
+                logger.info(f'input {c} - quitting application')
+                sys.exit()
+            else:
+                logger.error(f'invalid input - {c}')
+                return self._get_download_range(ch_nums)
         else:
-            logger.error(f'invalid input - {c}')
-            return self._get_download_range(ch_nums)
+            print(
+                '[a] - download all    [r] - select custom range    [s] - search another manga    [q] - quit app')
+            c = getch()
+
+            if c.lower() == 'a':
+                return {str(_) for _ in ch_nums}
+            elif c.lower() == 'r':
+                logger.info(f'input {c} - select custom range')
+                collect_range_input()
+            elif c.lower() == 's':
+                logger.warning(
+                    f'input {c} - abandoning the manga {self.title}')
+                from .mangodl import search_another
+                search_another()
+            elif c.lower() == 'q':
+                logger.info(f'input {c} - quitting application')
+                sys.exit()
+            else:
+                logger.error(f'invalid input - {c}')
+                return self._get_download_range(ch_nums)
 
         return s
 
@@ -281,32 +326,82 @@ class Manga:
         else:
             print(
                 f'Proceed to download {ch_count} chapter of {self.title}?')
-        print(
-            '[y] - yes, download    [r] - select range again    [s] - search another manga    [q] - quit app')
-        check = getch()
 
-        if check.lower() == 'y':
-            print('(~˘▾˘)~ okay, starting download now ~(˘▾˘~)')
-        elif check.lower() == 'r':
-            return self._display_chs()
-        elif check.lower() == 's':
-            logger.warning(f'abandoning manga -> {self.title}')
-            from .mangodl import search_another
-            search_another()
-        elif check.lower() == 'q':
-            logger.info(f'received input \'{check}\' - exiting program')
-            sys.exit()
+        if ARGS.url:
+            # don't print the 'search another manga' option
+            print(
+                '[y] - yes, download    [r] - select range again    [q] - quit app')
+            check = getch()
+
+            if check.lower() == 'y':
+                print('(~˘▾˘)~ okay, starting download now ~(˘▾˘~)')
+            elif check.lower() == 'r':
+                return self._display_chs()
+            elif check.lower() == 'q':
+                logger.info(f'received input \'{check}\' - exiting program')
+                sys.exit()
+            else:
+                logger.warning(f'invalid input - {check}')
+                return self._confirm_download()
         else:
-            logger.warning(f'invalid input - {check}')
-            return self._confirm_download()
+            print(
+                '[y] - yes, download    [r] - select range again    [s] - search another manga    [q] - quit app')
+            check = getch()
+
+            if check.lower() == 'y':
+                print('(~˘▾˘)~ okay, starting download now ~(˘▾˘~)')
+            elif check.lower() == 'r':
+                return self._display_chs()
+            elif check.lower() == 's':
+                logger.warning(f'abandoning manga -> {self.title}')
+                from .mangodl import search_another
+                search_another()
+            elif check.lower() == 'q':
+                logger.info(f'received input \'{check}\' - exiting program')
+                sys.exit()
+            else:
+                logger.warning(f'invalid input - {check}')
+                return self._confirm_download()
+
+    def handle_nameless(self, nameless_chs: List) -> None:
+        """Lets user decide what to do with chapters with no chapter number."""
+        print(
+            f'Found {len(nameless_chs)} chapter(s) with no chapter number ಠ_ಠ')
+        ch_titles = [ch['title'] for ch in nameless_chs]
+        print(f'Title(s):', ', '.join(ch_titles))
+        print('↑ download or ignore? ↑')
+        print('[y] - download as well    [n] - ignore')
+        c = getch()
+        if c.lower() == 'y':
+            logger.info(f'got input - {c}, will include in downloads')
+            self.s_downloads += nameless_chs
+        elif c.lower() == 'n':
+            logger.info(f'got input - {c}, will ignore')
+        else:
+            # reject input and try again
+            logger.error(f'got invalid input -{c}')
+            return self.handle_nameless(nameless_chs)
 
     def _compile_volume_info(self, vol_len: int) -> None:
         """Assigns a volume number to all downloaded mangas via their 
         respective Chapter instances."""
         logger.info('figuring out which chapter belongs to which volume')
+        #
+        # we will get a list of downloaded chapters from self.downloaded
+        # these will help us keep track of chapters and volumes
         orphans = []
         prelim_map = defaultdict(list)
 
+        for ch in self.downloaded:
+            if ch.ch_num == '_':
+                # bad news! a chapter without a chapter number
+                logger.warning(f'chapter id {ch.id} has no chapter number')
+            elif ch.vol_num == '':
+                orphans.append(ch)  # passed by reference
+            else:
+                prelim_map[ch.vol_num].append(ch.ch_num)
+        vol_nums = sorted(prelim_map)
+
         def fit_between():
             for i, orphan in enumerate(orphans[:]):
                 for j, vol_num in enumerate(vol_nums):
@@ -351,6 +446,7 @@ class Manga:
                     vol_nums.insert(0, new_vol_num)
                     logger.debug(f'creating volume {new_vol_num}')
                     for ch in new_vol:
+                        prelim_map[new_vol_num].append(ch.ch_num)
                         ch.vol_num = new_vol_num
                         logger.debug(
                             f'chapter {ch.ch_num} -> volume {new_vol_num}')
@@ -360,6 +456,7 @@ class Manga:
                     vol_nums.append(new_vol_num)
                     logger.debug(f'creating volume {new_vol_num}')
                     for ch in new_vol:
+                        prelim_map[new_vol_num].append(ch.ch_num)
                         ch.vol_num = new_vol_num
                         logger.debug(
                             f'chapter {ch.ch_num} -> volume {new_vol_num}')
@@ -369,19 +466,14 @@ class Manga:
                 f'no volume info found - defaulting to {vol_len} chapters per volume')
             for new_vol in chunk(orphans, vol_len):
                 new_vol_num = len(vol_nums) + 1
+                logger.info(f'creating new volume - vol. {new_vol_num}')
                 vol_nums.append(new_vol_num)
                 for ch in new_vol:
+                    prelim_map[new_vol_num].append(ch.ch_num)
                     ch.vol_num = new_vol_num
                     logger.debug(
                         f'chapter {ch.ch_num} -> volume {new_vol_num}')
 
-        for ch in self.downloaded:
-            if ch.vol_num == '':
-                orphans.append(ch)  # passed by reference
-            else:
-                prelim_map[ch.vol_num].append(ch.ch_num)
-        vol_nums = sorted(prelim_map)
-
         if orphans and vol_nums:
             fit_between()
             if orphans:
